#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
A collection of compression algorithms. The unit `refinery.decompress`
implements a brute force heuristic decompressor that attempts all known
algorithms against the input data.
"""
