
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
tlds = ['xn\\-\\-vermgensberatung\\-pwb',
 'xn\\-\\-vermgensberater\\-ctb',
 'xn\\-\\-clchc0ea0b2g2a9gcd',
 'xn\\-\\-w4r85el8fhu5dnra',
 'xn\\-\\-3oq18vl8pn36a',
 'xn\\-\\-5su34j936bgsg',
 'xn\\-\\-bck1b9a5dre4c',
 'xn\\-\\-mgbah1a3hjkrd',
 'xn\\-\\-mgbai9azgqp6j',
 'xn\\-\\-mgberp4a5d4ar',
 'xn\\-\\-xkc2dl3a5ee0h',
 'northwesternmutual',
 'travelersinsurance',
 'xn\\-\\-fzys8d69uvgm',
 'xn\\-\\-mgba7c0bbn0a',
 'xn\\-\\-xkc2al3hye2a',
 'xn\\-\\-i1b6b1a6a2e',
 'xn\\-\\-kcrx77d1x4a',
 'xn\\-\\-lgbbat1ad8j',
 'xn\\-\\-mgba3a4f16a',
 'xn\\-\\-mgbaakc7dvf',
 'xn\\-\\-mgbc0a9azcg',
 'xn\\-\\-nqv7fs00ema',
 'xn\\-\\-54b7fta0cc',
 'xn\\-\\-6qq986b3xl',
 'xn\\-\\-80aqecdr1a',
 'xn\\-\\-b4w605ferd',
 'xn\\-\\-fiq228c5hs',
 'xn\\-\\-h2breg3eve',
 'xn\\-\\-jlq61u9w7b',
 'xn\\-\\-mgba3a3ejt',
 'xn\\-\\-mgbaam7a8h',
 'xn\\-\\-mgbayh7gpa',
 'xn\\-\\-mgbbh1a71e',
 'xn\\-\\-mgbca7dzdo',
 'xn\\-\\-mgbi4ecexp',
 'xn\\-\\-mgbx4cd0ab',
 'xn\\-\\-rvc1e0am3e',
 'americanexpress',
 'kerryproperties',
 'sandvikcoromant',
 'xn\\-\\-eckvdtc9d',
 'xn\\-\\-fpcrj9c3d',
 'xn\\-\\-fzc2c9e2c',
 'xn\\-\\-h2brj9c8c',
 'xn\\-\\-tiq49xqyj',
 'xn\\-\\-yfro4i67o',
 'xn\\-\\-ygbi2ammx',
 'afamilycompany',
 'americanfamily',
 'bananarepublic',
 'cancerresearch',
 'cookingchannel',
 'kerrylogistics',
 'weatherchannel',
 'xn\\-\\-3e0b707e',
 'xn\\-\\-45br5cyl',
 'xn\\-\\-80adxhks',
 'xn\\-\\-80asehdb',
 'xn\\-\\-8y0a063a',
 'xn\\-\\-gckr3f0f',
 'xn\\-\\-mgb9awbf',
 'xn\\-\\-mgbab2bd',
 'xn\\-\\-mgbgu82a',
 'xn\\-\\-mgbpl2fh',
 'xn\\-\\-mgbt3dhd',
 'xn\\-\\-mk1bu44c',
 'xn\\-\\-ngbc5azd',
 'xn\\-\\-ngbe9e0a',
 'xn\\-\\-ogbpf8fl',
 'xn\\-\\-qcka1pmc',
 'international',
 'lifeinsurance',
 'spreadbetting',
 'travelchannel',
 'wolterskluwer',
 'xn\\-\\-11b4c3d',
 'xn\\-\\-1ck2e1b',
 'xn\\-\\-1qqw23a',
 'xn\\-\\-2scrj9c',
 'xn\\-\\-3bst00m',
 'xn\\-\\-3ds443g',
 'xn\\-\\-3hcrj9c',
 'xn\\-\\-42c2d9a',
 'xn\\-\\-45brj9c',
 'xn\\-\\-55qw42g',
 'xn\\-\\-6frz82g',
 'xn\\-\\-80ao21a',
 'xn\\-\\-9krt00a',
 'xn\\-\\-cck2b3b',
 'xn\\-\\-czr694b',
 'xn\\-\\-d1acj3b',
 'xn\\-\\-efvy88h',
 'xn\\-\\-estv75g',
 'xn\\-\\-fct429k',
 'xn\\-\\-fjq720a',
 'xn\\-\\-flw351e',
 'xn\\-\\-g2xx48c',
 'xn\\-\\-gecrj9c',
 'xn\\-\\-gk3at1e',
 'xn\\-\\-h2brj9c',
 'xn\\-\\-hxt814e',
 'xn\\-\\-imr513n',
 'xn\\-\\-j6w193g',
 'xn\\-\\-jvr189m',
 'xn\\-\\-kprw13d',
 'xn\\-\\-kpry57d',
 'xn\\-\\-kpu716f',
 'xn\\-\\-mgbbh1a',
 'xn\\-\\-mgbtx2b',
 'xn\\-\\-mix891f',
 'xn\\-\\-nyqy26a',
 'xn\\-\\-otu796d',
 'xn\\-\\-pbt977c',
 'xn\\-\\-pgbs0dh',
 'xn\\-\\-q9jyb4c',
 'xn\\-\\-rhqv96g',
 'xn\\-\\-rovu88b',
 'xn\\-\\-s9brj9c',
 'xn\\-\\-ses554g',
 'xn\\-\\-t60b56a',
 'xn\\-\\-vuq861b',
 'xn\\-\\-w4rs40l',
 'xn\\-\\-xhq521b',
 'xn\\-\\-zfr164b',
 'construction',
 'lplfinancial',
 'scholarships',
 'versicherung',
 'xn\\-\\-30rr7y',
 'xn\\-\\-3pxu8k',
 'xn\\-\\-45q11c',
 'xn\\-\\-4gbrim',
 'xn\\-\\-55qx5d',
 'xn\\-\\-5tzm5g',
 'xn\\-\\-80aswg',
 'xn\\-\\-90a3ac',
 'xn\\-\\-9dbq2a',
 'xn\\-\\-9et52u',
 'xn\\-\\-c2br7g',
 'xn\\-\\-cg4bki',
 'xn\\-\\-czrs0t',
 'xn\\-\\-czru2d',
 'xn\\-\\-fiq64b',
 'xn\\-\\-fiqs8s',
 'xn\\-\\-fiqz9s',
 'xn\\-\\-io0a7i',
 'xn\\-\\-kput3i',
 'xn\\-\\-mxtq1m',
 'xn\\-\\-o3cw4h',
 'xn\\-\\-pssy2u',
 'xn\\-\\-unup4y',
 'xn\\-\\-wgbh1c',
 'xn\\-\\-wgbl6a',
 'xn\\-\\-y9a3aq',
 'accountants',
 'barclaycard',
 'blackfriday',
 'blockbuster',
 'bridgestone',
 'calvinklein',
 'contractors',
 'creditunion',
 'engineering',
 'enterprises',
 'foodnetwork',
 'investments',
 'kerryhotels',
 'lamborghini',
 'motorcycles',
 'olayangroup',
 'photography',
 'playstation',
 'productions',
 'progressive',
 'redumbrella',
 'rightathome',
 'williamhill',
 'xn\\-\\-90ais',
 'xn\\-\\-c1avg',
 'xn\\-\\-d1alf',
 'xn\\-\\-e1a4c',
 'xn\\-\\-fhbei',
 'xn\\-\\-j1aef',
 'xn\\-\\-j1amh',
 'xn\\-\\-l1acc',
 'xn\\-\\-ngbrx',
 'xn\\-\\-nqv7f',
 'xn\\-\\-p1acf',
 'xn\\-\\-qxa6a',
 'xn\\-\\-tckwe',
 'xn\\-\\-vhquv',
 'accountant',
 'apartments',
 'associates',
 'basketball',
 'bnpparibas',
 'boehringer',
 'capitalone',
 'consulting',
 'creditcard',
 'cuisinella',
 'eurovision',
 'extraspace',
 'foundation',
 'healthcare',
 'immobilien',
 'industries',
 'management',
 'mitsubishi',
 'nationwide',
 'newholland',
 'nextdirect',
 'onyourside',
 'properties',
 'protection',
 'prudential',
 'realestate',
 'republican',
 'restaurant',
 'schaeffler',
 'swiftcover',
 'tatamotors',
 'technology',
 'university',
 'vistaprint',
 'vlaanderen',
 'volkswagen',
 'xn\\-\\-90ae',
 'xn\\-\\-node',
 'xn\\-\\-p1ai',
 'xn\\-\\-qxam',
 'accenture',
 'alfaromeo',
 'allfinanz',
 'amsterdam',
 'analytics',
 'aquarelle',
 'barcelona',
 'bloomberg',
 'christmas',
 'community',
 'directory',
 'education',
 'equipment',
 'fairwinds',
 'financial',
 'firestone',
 'fresenius',
 'frontdoor',
 'fujixerox',
 'furniture',
 'goldpoint',
 'hisamitsu',
 'homedepot',
 'homegoods',
 'homesense',
 'institute',
 'insurance',
 'kuokgroup',
 'lancaster',
 'landrover',
 'lifestyle',
 'marketing',
 'marshalls',
 'melbourne',
 'microsoft',
 'panasonic',
 'passagens',
 'pramerica',
 'richardli',
 'scjohnson',
 'shangrila',
 'solutions',
 'statebank',
 'statefarm',
 'stockholm',
 'travelers',
 'vacations',
 'yodobashi',
 'abudhabi',
 'airforce',
 'allstate',
 'attorney',
 'barclays',
 'barefoot',
 'bargains',
 'baseball',
 'boutique',
 'bradesco',
 'broadway',
 'brussels',
 'budapest',
 'builders',
 'business',
 'capetown',
 'catering',
 'catholic',
 'cipriani',
 'cityeats',
 'cleaning',
 'clinique',
 'clothing',
 'commbank',
 'computer',
 'delivery',
 'deloitte',
 'democrat',
 'diamonds',
 'discount',
 'discover',
 'download',
 'engineer',
 'ericsson',
 'esurance',
 'etisalat',
 'exchange',
 'feedback',
 'fidelity',
 'firmdale',
 'football',
 'frontier',
 'goodyear',
 'grainger',
 'graphics',
 'guardian',
 'hdfcbank',
 'helsinki',
 'holdings',
 'hospital',
 'infiniti',
 'ipiranga',
 'istanbul',
 'jpmorgan',
 'lighting',
 'lundbeck',
 'marriott',
 'maserati',
 'mckinsey',
 'memorial',
 'merckmsd',
 'mortgage',
 'observer',
 'partners',
 'pharmacy',
 'pictures',
 'plumbing',
 'property',
 'redstone',
 'reliance',
 'saarland',
 'samsclub',
 'security',
 'services',
 'shopping',
 'showtime',
 'softbank',
 'software',
 'stcgroup',
 'supplies',
 'symantec',
 'training',
 'vanguard',
 'ventures',
 'verisign',
 'woodside',
 'yokohama',
 'abogado',
 'academy',
 'agakhan',
 'alibaba',
 'android',
 'athleta',
 'auction',
 'audible',
 'auspost',
 'avianca',
 'banamex',
 'bauhaus',
 'bentley',
 'bestbuy',
 'booking',
 'brother',
 'bugatti',
 'capital',
 'caravan',
 'careers',
 'channel',
 'charity',
 'chintai',
 'citadel',
 'clubmed',
 'college',
 'cologne',
 'comcast',
 'company',
 'compare',
 'contact',
 'cooking',
 'corsica',
 'country',
 'coupons',
 'courses',
 'cricket',
 'cruises',
 'dentist',
 'digital',
 'domains',
 'exposed',
 'express',
 'farmers',
 'fashion',
 'ferrari',
 'ferrero',
 'finance',
 'fishing',
 'fitness',
 'flights',
 'florist',
 'flowers',
 'forsale',
 'frogans',
 'fujitsu',
 'gallery',
 'genting',
 'godaddy',
 'grocery',
 'guitars',
 'hamburg',
 'hangout',
 'hitachi',
 'holiday',
 'hosting',
 'hoteles',
 'hotmail',
 'hyundai',
 'ismaili',
 'jewelry',
 'juniper',
 'kitchen',
 'komatsu',
 'lacaixa',
 'lanxess',
 'lasalle',
 'latrobe',
 'leclerc',
 'limited',
 'lincoln',
 'markets',
 'metlife',
 'monster',
 'netbank',
 'netflix',
 'network',
 'neustar',
 'okinawa',
 'oldnavy',
 'organic',
 'origins',
 'philips',
 'pioneer',
 'politie',
 'realtor',
 'recipes',
 'rentals',
 'reviews',
 'rexroth',
 'samsung',
 'sandvik',
 'schmidt',
 'schwarz',
 'science',
 'shiksha',
 'shriram',
 'singles',
 'staples',
 'storage',
 'support',
 'surgery',
 'systems',
 'temasek',
 'theater',
 'theatre',
 'tickets',
 'tiffany',
 'toshiba',
 'trading',
 'walmart',
 'wanggou',
 'watches',
 'weather',
 'website',
 'wedding',
 'whoswho',
 'windows',
 'winners',
 'xfinity',
 'yamaxun',
 'youtube',
 'zuerich',
 'abarth',
 'abbott',
 'abbvie',
 'africa',
 'agency',
 'airbus',
 'airtel',
 'alipay',
 'alsace',
 'alstom',
 'anquan',
 'aramco',
 'author',
 'bayern',
 'beauty',
 'berlin',
 'bharti',
 'bostik',
 'boston',
 'broker',
 'camera',
 'career',
 'caseih',
 'casino',
 'center',
 'chanel',
 'chrome',
 'church',
 'circle',
 'claims',
 'clinic',
 'coffee',
 'comsec',
 'condos',
 'coupon',
 'credit',
 'cruise',
 'dating',
 'datsun',
 'dealer',
 'degree',
 'dental',
 'design',
 'direct',
 'doctor',
 'dunlop',
 'dupont',
 'durban',
 'emerck',
 'energy',
 'estate',
 'events',
 'expert',
 'family',
 'flickr',
 'futbol',
 'gallup',
 'garden',
 'george',
 'giving',
 'global',
 'google',
 'gratis',
 'health',
 'hermes',
 'hiphop',
 'hockey',
 'hotels',
 'hughes',
 'imamat',
 'insure',
 'intuit',
 'jaguar',
 'joburg',
 'juegos',
 'kaufen',
 'kinder',
 'kindle',
 'kosher',
 'lancia',
 'latino',
 'lawyer',
 'lefrak',
 'living',
 'locker',
 'london',
 'luxury',
 'madrid',
 'maison',
 'makeup',
 'market',
 'mattel',
 'mobile',
 'monash',
 'mormon',
 'moscow',
 'museum',
 'mutual',
 'nagoya',
 'natura',
 'nissan',
 'nissay',
 'norton',
 'nowruz',
 'office',
 'olayan',
 'online',
 'oracle',
 'orange',
 'otsuka',
 'pfizer',
 'photos',
 'physio',
 'pictet',
 'quebec',
 'racing',
 'realty',
 'reisen',
 'repair',
 'report',
 'review',
 'rocher',
 'rogers',
 'ryukyu',
 'safety',
 'sakura',
 'sanofi',
 'school',
 'schule',
 'search',
 'secure',
 'select',
 'shouji',
 'soccer',
 'social',
 'stream',
 'studio',
 'supply',
 'suzuki',
 'swatch',
 'sydney',
 'taipei',
 'taobao',
 'target',
 'tattoo',
 'tennis',
 'tienda',
 'tjmaxx',
 'tkmaxx',
 'toyota',
 'travel',
 'unicom',
 'viajes',
 'viking',
 'villas',
 'virgin',
 'vision',
 'voting',
 'voyage',
 'vuelos',
 'walter',
 'webcam',
 'xihuan',
 'yachts',
 'yandex',
 'zappos',
 'actor',
 'adult',
 'aetna',
 'amfam',
 'amica',
 'apple',
 'archi',
 'audio',
 'autos',
 'azure',
 'baidu',
 'beats',
 'bible',
 'bingo',
 'black',
 'boats',
 'bosch',
 'build',
 'canon',
 'cards',
 'chase',
 'cheap',
 'cisco',
 'citic',
 'click',
 'cloud',
 'coach',
 'codes',
 'crown',
 'cymru',
 'dabur',
 'dance',
 'deals',
 'delta',
 'drive',
 'dubai',
 'earth',
 'edeka',
 'email',
 'epson',
 'faith',
 'fedex',
 'final',
 'forex',
 'forum',
 'gallo',
 'games',
 'gifts',
 'gives',
 'glade',
 'glass',
 'globo',
 'gmail',
 'green',
 'gripe',
 'group',
 'gucci',
 'guide',
 'homes',
 'honda',
 'horse',
 'house',
 'hyatt',
 'ikano',
 'intel',
 'irish',
 'iveco',
 'jetzt',
 'koeln',
 'kyoto',
 'lamer',
 'lease',
 'legal',
 'lexus',
 'lilly',
 'linde',
 'lipsy',
 'lixil',
 'loans',
 'locus',
 'lotte',
 'lotto',
 'lupin',
 'macys',
 'mango',
 'media',
 'miami',
 'money',
 'movie',
 'nadex',
 'nexus',
 'nikon',
 'ninja',
 'nokia',
 'nowtv',
 'omega',
 'onion',
 'osaka',
 'paris',
 'parts',
 'party',
 'phone',
 'photo',
 'pizza',
 'place',
 'poker',
 'praxi',
 'press',
 'prime',
 'promo',
 'quest',
 'radio',
 'rehab',
 'reise',
 'ricoh',
 'rocks',
 'rodeo',
 'rugby',
 'salon',
 'sener',
 'seven',
 'sharp',
 'shell',
 'shoes',
 'skype',
 'sling',
 'smart',
 'smile',
 'solar',
 'space',
 'sport',
 'stada',
 'store',
 'study',
 'style',
 'sucks',
 'swiss',
 'tatar',
 'tires',
 'tirol',
 'tmall',
 'today',
 'tokyo',
 'tools',
 'toray',
 'total',
 'tours',
 'trade',
 'trust',
 'tunes',
 'tushu',
 'ubank',
 'vegas',
 'video',
 'vodka',
 'volvo',
 'wales',
 'watch',
 'weber',
 'weibo',
 'works',
 'world',
 'xerox',
 'yahoo',
 'aarp',
 'able',
 'adac',
 'aero',
 'aigo',
 'akdn',
 'ally',
 'amex',
 'arab',
 'army',
 'arpa',
 'arte',
 'asda',
 'asia',
 'audi',
 'auto',
 'baby',
 'band',
 'bank',
 'bbva',
 'beer',
 'best',
 'bike',
 'bing',
 'blog',
 'blue',
 'bofa',
 'bond',
 'book',
 'buzz',
 'cafe',
 'call',
 'camp',
 'care',
 'cars',
 'casa',
 'case',
 'cash',
 'cbre',
 'cern',
 'chat',
 'citi',
 'city',
 'club',
 'cool',
 'coop',
 'cyou',
 'data',
 'date',
 'dclk',
 'deal',
 'dell',
 'desi',
 'diet',
 'dish',
 'docs',
 'duck',
 'dvag',
 'erni',
 'fage',
 'fail',
 'fans',
 'farm',
 'fast',
 'fiat',
 'fido',
 'film',
 'fire',
 'fish',
 'flir',
 'food',
 'ford',
 'free',
 'fund',
 'game',
 'gbiz',
 'gent',
 'ggee',
 'gift',
 'gmbh',
 'gold',
 'golf',
 'goog',
 'guge',
 'guru',
 'hair',
 'haus',
 'hdfc',
 'help',
 'here',
 'hgtv',
 'host',
 'hsbc',
 'icbc',
 'ieee',
 'imdb',
 'immo',
 'info',
 'itau',
 'java',
 'jeep',
 'jobs',
 'jprs',
 'kddi',
 'kiwi',
 'kpmg',
 'kred',
 'land',
 'lego',
 'lgbt',
 'lidl',
 'life',
 'like',
 'limo',
 'link',
 'live',
 'loan',
 'loft',
 'love',
 'ltda',
 'luxe',
 'maif',
 'meet',
 'meme',
 'menu',
 'mini',
 'mint',
 'mobi',
 'moda',
 'moto',
 'name',
 'navy',
 'news',
 'next',
 'nico',
 'nike',
 'ollo',
 'open',
 'page',
 'pars',
 'pccw',
 'pics',
 'ping',
 'pink',
 'play',
 'plus',
 'pohl',
 'porn',
 'post',
 'prod',
 'prof',
 'qpon',
 'raid',
 'read',
 'reit',
 'rent',
 'rest',
 'rich',
 'rmit',
 'room',
 'rsvp',
 'ruhr',
 'safe',
 'sale',
 'sarl',
 'save',
 'saxo',
 'scor',
 'scot',
 'seat',
 'seek',
 'sexy',
 'shaw',
 'shia',
 'shop',
 'show',
 'silk',
 'sina',
 'site',
 'skin',
 'sncf',
 'sohu',
 'song',
 'sony',
 'spot',
 'star',
 'surf',
 'talk',
 'taxi',
 'team',
 'tech',
 'teva',
 'tiaa',
 'tips',
 'town',
 'toys',
 'tube',
 'vana',
 'visa',
 'viva',
 'vivo',
 'vote',
 'voto',
 'wang',
 'weir',
 'wien',
 'wiki',
 'wine',
 'work',
 'xbox',
 'yoga',
 'zara',
 'zero',
 'zone',
 'aaa',
 'abb',
 'abc',
 'aco',
 'ads',
 'aeg',
 'afl',
 'aig',
 'anz',
 'aol',
 'app',
 'art',
 'aws',
 'axa',
 'bar',
 'bbc',
 'bbt',
 'bcg',
 'bcn',
 'bet',
 'bid',
 'bio',
 'bit',
 'biz',
 'bms',
 'bmw',
 'bom',
 'boo',
 'bot',
 'box',
 'buy',
 'bzh',
 'cab',
 'cal',
 'cam',
 'car',
 'cat',
 'cba',
 'cbn',
 'cbs',
 'ceb',
 'ceo',
 'cfa',
 'cfd',
 'com',
 'cpa',
 'crs',
 'csc',
 'dad',
 'day',
 'dds',
 'dev',
 'dhl',
 'diy',
 'dnp',
 'dog',
 'dot',
 'dtv',
 'dvr',
 'eat',
 'eco',
 'edu',
 'esq',
 'eus',
 'fan',
 'fit',
 'fly',
 'foo',
 'fox',
 'frl',
 'ftr',
 'fun',
 'fyi',
 'gal',
 'gap',
 'gay',
 'gdn',
 'gea',
 'gle',
 'gmo',
 'gmx',
 'goo',
 'gop',
 'got',
 'gov',
 'hbo',
 'hiv',
 'hkt',
 'hot',
 'how',
 'ibm',
 'ice',
 'icu',
 'ifm',
 'inc',
 'ing',
 'ink',
 'int',
 'ist',
 'itv',
 'jcb',
 'jcp',
 'jio',
 'jll',
 'jmp',
 'jnj',
 'jot',
 'joy',
 'kfh',
 'kia',
 'kim',
 'kpn',
 'krd',
 'lat',
 'law',
 'lds',
 'llc',
 'llp',
 'lol',
 'lpl',
 'ltd',
 'man',
 'map',
 'mba',
 'med',
 'men',
 'mil',
 'mit',
 'mlb',
 'mls',
 'mma',
 'moe',
 'moi',
 'mom',
 'mov',
 'msd',
 'mtn',
 'mtr',
 'nab',
 'nba',
 'nec',
 'net',
 'new',
 'nfl',
 'ngo',
 'nhk',
 'now',
 'nra',
 'nrw',
 'ntt',
 'nyc',
 'obi',
 'off',
 'one',
 'ong',
 'onl',
 'ooo',
 'org',
 'ott',
 'ovh',
 'pay',
 'pet',
 'phd',
 'pid',
 'pin',
 'pnc',
 'pro',
 'pru',
 'pub',
 'pwc',
 'qvc',
 'red',
 'ren',
 'ril',
 'rio',
 'rip',
 'run',
 'rwe',
 'sap',
 'sas',
 'sbi',
 'sbs',
 'sca',
 'scb',
 'ses',
 'sew',
 'sex',
 'sfr',
 'ski',
 'sky',
 'soy',
 'srl',
 'stc',
 'tab',
 'tax',
 'tci',
 'tdk',
 'tel',
 'thd',
 'tjx',
 'top',
 'trv',
 'tui',
 'tvs',
 'ubs',
 'uno',
 'uol',
 'ups',
 'vet',
 'vig',
 'vin',
 'vip',
 'wed',
 'win',
 'wme',
 'wow',
 'wtc',
 'wtf',
 'xin',
 'xxx',
 'xyz',
 'you',
 'yun',
 'zip',
 'ac',
 'ad',
 'ae',
 'af',
 'ag',
 'ai',
 'al',
 'am',
 'ao',
 'aq',
 'ar',
 'as',
 'at',
 'au',
 'aw',
 'ax',
 'az',
 'ba',
 'bb',
 'bd',
 'be',
 'bf',
 'bg',
 'bh',
 'bi',
 'bj',
 'bm',
 'bn',
 'bo',
 'br',
 'bs',
 'bt',
 'bv',
 'bw',
 'by',
 'bz',
 'ca',
 'cc',
 'cd',
 'cf',
 'cg',
 'ch',
 'ci',
 'ck',
 'cl',
 'cm',
 'cn',
 'co',
 'cr',
 'cu',
 'cv',
 'cw',
 'cx',
 'cy',
 'cz',
 'de',
 'dj',
 'dk',
 'dm',
 'do',
 'dz',
 'ec',
 'ee',
 'eg',
 'er',
 'es',
 'et',
 'eu',
 'fi',
 'fj',
 'fk',
 'fm',
 'fo',
 'fr',
 'ga',
 'gb',
 'gd',
 'ge',
 'gf',
 'gg',
 'gh',
 'gi',
 'gl',
 'gm',
 'gn',
 'gp',
 'gq',
 'gr',
 'gs',
 'gt',
 'gu',
 'gw',
 'gy',
 'hk',
 'hm',
 'hn',
 'hr',
 'ht',
 'hu',
 'id',
 'ie',
 'il',
 'im',
 'in',
 'io',
 'iq',
 'ir',
 'is',
 'it',
 'je',
 'jm',
 'jo',
 'jp',
 'ke',
 'kg',
 'kh',
 'ki',
 'km',
 'kn',
 'kp',
 'kr',
 'kw',
 'ky',
 'kz',
 'la',
 'lb',
 'lc',
 'li',
 'lk',
 'lr',
 'ls',
 'lt',
 'lu',
 'lv',
 'ly',
 'ma',
 'mc',
 'md',
 'me',
 'mg',
 'mh',
 'mk',
 'ml',
 'mm',
 'mn',
 'mo',
 'mp',
 'mq',
 'mr',
 'ms',
 'mt',
 'mu',
 'mv',
 'mw',
 'mx',
 'my',
 'mz',
 'na',
 'nc',
 'ne',
 'nf',
 'ng',
 'ni',
 'nl',
 'no',
 'np',
 'nr',
 'nu',
 'nz',
 'om',
 'pa',
 'pe',
 'pf',
 'pg',
 'ph',
 'pk',
 'pl',
 'pm',
 'pn',
 'pr',
 'ps',
 'pt',
 'pw',
 'py',
 'qa',
 're',
 'ro',
 'rs',
 'ru',
 'rw',
 'sa',
 'sb',
 'sc',
 'sd',
 'se',
 'sg',
 'sh',
 'si',
 'sj',
 'sk',
 'sl',
 'sm',
 'sn',
 'so',
 'sr',
 'ss',
 'st',
 'su',
 'sv',
 'sx',
 'sy',
 'sz',
 'tc',
 'td',
 'tf',
 'tg',
 'th',
 'tj',
 'tk',
 'tl',
 'tm',
 'tn',
 'to',
 'tr',
 'tt',
 'tv',
 'tw',
 'tz',
 'ua',
 'ug',
 'uk',
 'us',
 'uy',
 'uz',
 'va',
 'vc',
 've',
 'vg',
 'vi',
 'vn',
 'vu',
 'wf',
 'ws',
 'ye',
 'yt',
 'za',
 'zm',
 'zw']
