#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ... import RefineryPartialResult
from . import KeyDerivation

from Crypto.Cipher import DES

__all__ = ['DESDerive']


class DESDerive(KeyDerivation):
    """
    Implements the same functionality as `DES_string_to_key` in OpenSSL. It
    converts a string to an 8 byte DES key with odd byte parity, per FIPS
    specification. This is not a modern key derivation function.
    """

    _DEFAULT_SIZE = 8

    _TABLE = bytearray((
        0x01, 0x01, 0x02, 0x02, 0x04, 0x04, 0x07, 0x07,
        0x08, 0x08, 0x0B, 0x0B, 0x0D, 0x0D, 0x0E, 0x0E,
        0x10, 0x10, 0x13, 0x13, 0x15, 0x15, 0x16, 0x16,
        0x19, 0x19, 0x1A, 0x1A, 0x1C, 0x1C, 0x1F, 0x1F,
        0x20, 0x20, 0x23, 0x23, 0x25, 0x25, 0x26, 0x26,
        0x29, 0x29, 0x2A, 0x2A, 0x2C, 0x2C, 0x2F, 0x2F,
        0x31, 0x31, 0x32, 0x32, 0x34, 0x34, 0x37, 0x37,
        0x38, 0x38, 0x3B, 0x3B, 0x3D, 0x3D, 0x3E, 0x3E,
        0x40, 0x40, 0x43, 0x43, 0x45, 0x45, 0x46, 0x46,
        0x49, 0x49, 0x4A, 0x4A, 0x4C, 0x4C, 0x4F, 0x4F,
        0x51, 0x51, 0x52, 0x52, 0x54, 0x54, 0x57, 0x57,
        0x58, 0x58, 0x5B, 0x5B, 0x5D, 0x5D, 0x5E, 0x5E,
        0x61, 0x61, 0x62, 0x62, 0x64, 0x64, 0x67, 0x67,
        0x68, 0x68, 0x6B, 0x6B, 0x6D, 0x6D, 0x6E, 0x6E,
        0x70, 0x70, 0x73, 0x73, 0x75, 0x75, 0x76, 0x76,
        0x79, 0x79, 0x7A, 0x7A, 0x7C, 0x7C, 0x7F, 0x7F,
        0x80, 0x80, 0x83, 0x83, 0x85, 0x85, 0x86, 0x86,
        0x89, 0x89, 0x8A, 0x8A, 0x8C, 0x8C, 0x8F, 0x8F,
        0x91, 0x91, 0x92, 0x92, 0x94, 0x94, 0x97, 0x97,
        0x98, 0x98, 0x9B, 0x9B, 0x9D, 0x9D, 0x9E, 0x9E,
        0xA1, 0xA1, 0xA2, 0xA2, 0xA4, 0xA4, 0xA7, 0xA7,
        0xA8, 0xA8, 0xAB, 0xAB, 0xAD, 0xAD, 0xAE, 0xAE,
        0xB0, 0xB0, 0xB3, 0xB3, 0xB5, 0xB5, 0xB6, 0xB6,
        0xB9, 0xB9, 0xBA, 0xBA, 0xBC, 0xBC, 0xBF, 0xBF,
        0xC1, 0xC1, 0xC2, 0xC2, 0xC4, 0xC4, 0xC7, 0xC7,
        0xC8, 0xC8, 0xCB, 0xCB, 0xCD, 0xCD, 0xCE, 0xCE,
        0xD0, 0xD0, 0xD3, 0xD3, 0xD5, 0xD5, 0xD6, 0xD6,
        0xD9, 0xD9, 0xDA, 0xDA, 0xDC, 0xDC, 0xDF, 0xDF,
        0xE0, 0xE0, 0xE3, 0xE3, 0xE5, 0xE5, 0xE6, 0xE6,
        0xE9, 0xE9, 0xEA, 0xEA, 0xEC, 0xEC, 0xEF, 0xEF,
        0xF1, 0xF1, 0xF2, 0xF2, 0xF4, 0xF4, 0xF7, 0xF7,
        0xF8, 0xF8, 0xFB, 0xFB, 0xFD, 0xFD, 0xFE, 0xFE
    ))

    def process(self, password):
        key = bytearray(8)

        for i, c in enumerate(password):
            if ((i % 16) < 8):
                key[i % 8] ^= (c << 1) & 0xFF
            else:
                c = (((c << 4) & 0xf0) | ((c >> 4) & 0x0f))
                c = (((c << 2) & 0xcc) | ((c >> 2) & 0x33))
                c = (((c << 1) & 0xaa) | ((c >> 1) & 0x55))
                key[7 - (i % 8)] ^= c

        n = len(password)

        key[:] = (self._TABLE[b] for b in key)
        des = DES.new(key, DES.MODE_CBC, iv=key)
        key = des.decrypt(password.ljust(n + 8 - n % 8, b'\0'))[-8:]
        key = bytes((self._TABLE[b] for b in key))

        if self.args.size > 8:
            raise RefineryPartialResult('DESDerive can provide at most 8 bytes.', partial=key)
        return key[:self.args.size]
